@@ -296,10 +296,7 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 		// Clone the image and the alpha
 		int image_size = mlt_image_format_size( self->format, self->current_width, self->current_height, NULL );
 		uint8_t *image_copy = mlt_pool_alloc( image_size );
-		// We use height-1 because mlt_image_format_size() uses height + 1.
-		// XXX Remove -1 when mlt_image_format_size() is changed.
-		memcpy( image_copy, self->current_image,
-			mlt_image_format_size( self->format, self->current_width, self->current_height - 1, NULL ) );
+		memcpy( image_copy, self->current_image, image_size );
 		// Now update properties so we free the copy after
 		mlt_frame_set_image( frame, image_copy, image_size, mlt_pool_release );
 		// We're going to pass the copy on
